@@ -1,12 +1,15 @@
+import logging
 import secrets
 
+from django.conf import settings
 from django.db import IntegrityError
 
 from django_filters import rest_framework as filters
 from rest_framework.filters import SearchFilter
+from drf_rw_serializers import viewsets as rw_viewsets
 from drf_spectacular.types import OpenApiTypes
 from drf_spectacular.utils import extend_schema, extend_schema_view, OpenApiParameter
-from rest_framework import generics, mixins, serializers, status, viewsets
+from rest_framework import mixins, serializers, status, viewsets
 from rest_framework.decorators import action
 from rest_framework.exceptions import NotFound, ValidationError
 from rest_framework.generics import get_object_or_404
@@ -17,7 +20,9 @@ from rest_framework.reverse import reverse
 
 from argus.auth.models import User
 from argus.drf.permissions import IsSuperuserOrReadOnly
+from argus.incident.ticket.base import TicketPluginException
 from argus.util.datetime_utils import INFINITY_REPR
+from argus.util.utils import import_class_from_dotted_path
 
 from .forms import AddSourceSystemForm
 from .filters import IncidentFilter, SourceLockedIncidentFilter
@@ -31,12 +36,16 @@ from .models import (
 )
 from .serializers import (
     UpdateAcknowledgementSerializer,
-    AcknowledgementSerializer,
+    EmptySerializer,
     EventSerializer,
     IncidentPureDeserializer,
     IncidentSerializer,
     IncidentTicketUrlSerializer,
+    RequestAcknowledgementSerializer,
     RequestBulkAcknowledgementSerializer,
+    RequestBulkEventSerializer,
+    RequestBulkTicketUrlSerializer,
+    ResponseAcknowledgementSerializer,
     ResponseBulkSerializer,
     SourceSystemSerializer,
     SourceSystemTypeSerializer,
@@ -44,6 +53,8 @@ from .serializers import (
     IncidentTagRelation,
 )
 
+LOG = logging.getLogger(__name__)
+
 
 # Used in OpenApiParameter
 BooleanStringOAEnum = ("true", "false")
@@ -248,6 +259,64 @@ class IncidentViewSet(
             return Response(serializer.errors, status=status.HTTP_400_BAD_REQUEST)
 
 
+@extend_schema_view(
+    update=extend_schema(
+        request=EmptySerializer,
+    ),
+)
+class TicketPluginViewSet(viewsets.ViewSet):
+    permission_classes = [IsAuthenticated]
+    serializer_class = IncidentTicketUrlSerializer
+    queryset = Incident.objects.all()
+
+    def update(self, request, incident_pk=None):
+        incident = get_object_or_404(self.queryset, pk=incident_pk)
+
+        if incident.ticket_url:
+            serializer = self.serializer_class(data={"ticket_url": incident.ticket_url})
+            if serializer.is_valid():
+                return Response(serializer.data, status=status.HTTP_200_OK)
+            return Response(serializer.errors, status=status.HTTP_400_BAD_REQUEST)
+
+        plugin = getattr(settings, "TICKET_PLUGIN", None)
+
+        if not plugin:
+            return Response(
+                data="No path to ticket plugin can be found in the settings. Please update the setting 'TICKET_PLUGIN'.",
+                status=status.HTTP_400_BAD_REQUEST,
+            )
+
+        try:
+            ticket_class = import_class_from_dotted_path(plugin)
+        except Exception:
+            LOG.exception("Could not import ticket plugin from path %s", plugin)
+            return Response(
+                data="Ticket plugins are incorrectly configured.",
+                status=status.HTTP_400_BAD_REQUEST,
+            )
+
+        try:
+            url = ticket_class.create_ticket(incident)
+        except TicketPluginException as e:
+            return Response(
+                data=str(e),
+                status=status.HTTP_400_BAD_REQUEST,
+            )
+
+        if url:
+            incident.ticket_url = url
+            incident.save(update_fields=["ticket_url"])
+            serializer = self.serializer_class(data={"ticket_url": incident.ticket_url})
+            if serializer.is_valid():
+                return Response(serializer.data, status=status.HTTP_200_OK)
+            return Response(serializer.errors, status=status.HTTP_400_BAD_REQUEST)
+
+        return Response(
+            data="No url could be generated. Please check that the ticket plugin provides a function to create tickets.",
+            status=status.HTTP_400_BAD_REQUEST,
+        )
+
+
 class IncidentTagViewSet(
     mixins.ListModelMixin,
     mixins.RetrieveModelMixin,
@@ -417,30 +486,29 @@ class EventViewSet(mixins.ListModelMixin, mixins.CreateModelMixin, mixins.Retrie
 
 
 @extend_schema_view(
+    create=extend_schema(
+        request=RequestAcknowledgementSerializer,
+        responses={"201": ResponseAcknowledgementSerializer},
+    ),
     update=extend_schema(
         request=UpdateAcknowledgementSerializer,
-        responses={"200": AcknowledgementSerializer},
+        responses={"200": ResponseAcknowledgementSerializer},
     ),
     partial_update=extend_schema(
         request=UpdateAcknowledgementSerializer,
-        responses={"200": AcknowledgementSerializer},
+        responses={"200": ResponseAcknowledgementSerializer},
     ),
 )
-class AcknowledgementViewSet(
-    mixins.ListModelMixin,
-    mixins.CreateModelMixin,
-    mixins.RetrieveModelMixin,
-    mixins.UpdateModelMixin,
-    viewsets.GenericViewSet,
-):
+class AcknowledgementViewSet(rw_viewsets.ModelViewSet):
     queryset = Incident.objects.none()  # For OpenAPI
     permission_classes = [IsAuthenticated]
-    serializer_class = AcknowledgementSerializer
+    serializer_class = ResponseAcknowledgementSerializer
+    read_serializer_class = ResponseAcknowledgementSerializer
 
-    def get_serializer_class(self):
+    def get_write_serializer_class(self):
         if self.action in ("partial_update", "update"):
             return UpdateAcknowledgementSerializer
-        return self.serializer_class
+        return RequestAcknowledgementSerializer
 
     def get_incident(self):
         incident_pk = self.kwargs["incident_pk"]
@@ -449,7 +517,7 @@ class AcknowledgementViewSet(
     def get_queryset(self):
         return self.get_incident().acks
 
-    def perform_create(self, serializer: AcknowledgementSerializer):
+    def perform_create(self, serializer: RequestAcknowledgementSerializer):
         user = self.request.user
         incident = self.get_incident()
         serializer.save(incident=incident, actor=user)
@@ -493,16 +561,126 @@ class BulkAcknowledgementViewSet(viewsets.ViewSet):
                 status_codes_seen.add(status.HTTP_400_BAD_REQUEST)
                 continue
 
-            event = Event.objects.create(
-                incident=incident,
+            ack = incident.create_ack(
                 actor=actor,
-                timestamp=ack_data["event"]["timestamp"],
-                type=Event.Type.ACKNOWLEDGE,
-                description=ack_data["event"]["description"],
+                timestamp=ack_data["timestamp"],
+                description=ack_data["description"],
+                expiration=ack_data["expiration"],
             )
-            ack = Acknowledgement.objects.create(event=event, expiration=ack_data["expiration"])
             changes[str(incident_id)] = {
-                "ack": AcknowledgementSerializer(instance=ack).to_representation(instance=ack),
+                "ack": ResponseAcknowledgementSerializer(instance=ack).to_representation(instance=ack),
+                "status": status.HTTP_201_CREATED,
+                "errors": None,
+            }
+            status_codes_seen.add(status.HTTP_201_CREATED)
+
+        all_bad = status_codes_seen == set((status.HTTP_400_BAD_REQUEST,))
+
+        return Response(
+            data={"changes": changes}, status=status.HTTP_400_BAD_REQUEST if all_bad else status.HTTP_201_CREATED
+        )
+
+
+@extend_schema_view(
+    create=extend_schema(
+        request=RequestBulkEventSerializer,
+        responses=ResponseBulkSerializer,
+    )
+)
+class BulkEventViewSet(viewsets.ViewSet):
+    permission_classes = [IsAuthenticated]
+    serializer_class = ResponseBulkSerializer
+    write_serializer_class = RequestBulkEventSerializer
+    queryset = Incident.objects.all()
+
+    def create(self, request):
+        serializer = self.write_serializer_class(data=request.data, context={"request": request})
+
+        if not serializer.is_valid():
+            return Response(data=serializer.errors, status=status.HTTP_400_BAD_REQUEST)
+
+        incident_ids = serializer.data["ids"]
+        event_data = serializer.data["event"]
+        actor = request.user
+
+        incidents = {i.id: i for i in self.queryset.filter(pk__in=incident_ids)}
+        changes = {}
+        status_codes_seen = set()
+
+        for incident_id in incident_ids:
+            incident = incidents.get(incident_id)
+
+            if not incident:
+                changes[str(incident_id)] = {
+                    "event": None,
+                    "status": status.HTTP_400_BAD_REQUEST,
+                    "errors": {"ids": f"Incident with id {incident_id} could not be found."},
+                }
+                status_codes_seen.add(status.HTTP_400_BAD_REQUEST)
+                continue
+
+            event = Event.objects.create(
+                incident=incident,
+                actor=actor,
+                timestamp=event_data["timestamp"],
+                type=event_data["type"],
+                description=event_data["description"],
+            )
+            changes[str(incident_id)] = {
+                "event": EventSerializer(instance=event).to_representation(instance=event),
+                "status": status.HTTP_201_CREATED,
+                "errors": None,
+            }
+            status_codes_seen.add(status.HTTP_201_CREATED)
+
+        all_bad = status_codes_seen == set((status.HTTP_400_BAD_REQUEST,))
+
+        return Response(
+            data={"changes": changes}, status=status.HTTP_400_BAD_REQUEST if all_bad else status.HTTP_201_CREATED
+        )
+
+
+@extend_schema_view(
+    create=extend_schema(
+        request=RequestBulkTicketUrlSerializer,
+        responses=ResponseBulkSerializer,
+    )
+)
+class BulkTicketUrlViewSet(viewsets.ViewSet):
+    permission_classes = [IsAuthenticated]
+    serializer_class = ResponseBulkSerializer
+    write_serializer_class = RequestBulkTicketUrlSerializer
+    queryset = Incident.objects.all()
+
+    def create(self, request):
+        serializer = self.write_serializer_class(data=request.data, context={"request": request})
+
+        if not serializer.is_valid():
+            return Response(data=serializer.errors, status=status.HTTP_400_BAD_REQUEST)
+
+        incident_ids = serializer.data["ids"]
+        ticket_url = serializer.data["ticket_url"]
+
+        incidents = {i.id: i for i in self.queryset.filter(pk__in=incident_ids)}
+        changes = {}
+        status_codes_seen = set()
+
+        for incident_id in incident_ids:
+            incident = incidents.get(incident_id)
+
+            if not incident:
+                changes[str(incident_id)] = {
+                    "ticket_url": None,
+                    "status": status.HTTP_400_BAD_REQUEST,
+                    "errors": {"ids": f"Incident with id {incident_id} could not be found."},
+                }
+                status_codes_seen.add(status.HTTP_400_BAD_REQUEST)
+                continue
+
+            incident.ticket_url = ticket_url
+            incident.save()
+            changes[str(incident_id)] = {
+                "ticket_url": ticket_url,
                 "status": status.HTTP_201_CREATED,
                 "errors": None,
             }
